@@ -3,7 +3,6 @@
 mkdir build
 cd build
 
-CMAKE_CXX_FLAGS="-std=c++17"
 if [[ "$CXX" == clang++* ]]; then
     CMAKE_CXX_FLAGS+=" -stdlib=libstdc++";
 fi
@@ -16,5 +15,5 @@ cmake \
   -DSNMALLOC_QEMU_WORKAROUND=ON \
   -DCMAKE_BUILD_TYPE=$BUILD_TYPE \
   -DCMAKE_CXX_FLAGS="$CMAKE_CXX_FLAGS" \
-  ..
+  ${SNMALLOC_SRC-..}
 ninja
\ No newline at end of file
