@@ -1,73 +1,31 @@
 #pragma once
 
 #include "helpers.h"
+#include "invalidptr.h"
+#include "ptrwrap.h"
 
 #include <cstdint>
 #include <type_traits>
 
 namespace snmalloc
 {
-  /**
-   * Invalid pointer class.  This is similar to `std::nullptr_t`, but allows
-   * other values.
-   */
-  template<address_t Sentinel>
-  struct InvalidPointer
-  {
-    /**
-     * Equality comparison. Two invalid pointer values with the same sentinel
-     * are always the same, invalid pointer values with different sentinels are
-     * always different.
-     */
-    template<address_t OtherSentinel>
-    constexpr bool operator==(const InvalidPointer<OtherSentinel>&)
-    {
-      return Sentinel == OtherSentinel;
-    }
-    /**
-     * Equality comparison. Two invalid pointer values with the same sentinel
-     * are always the same, invalid pointer values with different sentinels are
-     * always different.
-     */
-    template<address_t OtherSentinel>
-    constexpr bool operator!=(const InvalidPointer<OtherSentinel>&)
-    {
-      return Sentinel != OtherSentinel;
-    }
-    /**
-     * Implicit conversion, creates a pointer with the value of the sentinel.
-     * On CHERI and other provenance-tracking systems, this is a
-     * provenance-free integer and so will trap if dereferenced, on other
-     * systems the sentinel should be a value in unmapped memory.
-     */
-    template<typename T>
-    operator T*() const
-    {
-      return reinterpret_cast<T*>(Sentinel);
-    }
-    /**
-     * Implicit conversion to an address, returns the sentinel value.
-     */
-    operator address_t() const
-    {
-      return Sentinel;
-    }
-  };
-
   template<
     class T,
+    template<typename> typename Ptr = Pointer,
     class Terminator = std::nullptr_t,
-    void on_clear(T*) = ignore<T>>
+    void on_clear(Ptr<T>) = ignore<T, Ptr>>
   class DLList final
   {
   private:
     static_assert(
-      std::is_same<decltype(T::prev), T*>::value, "T->prev must be a T*");
+      std::is_same<decltype(T::prev), Ptr<T>>::value,
+      "T->prev must be a Ptr<T>");
     static_assert(
-      std::is_same<decltype(T::next), T*>::value, "T->next must be a T*");
+      std::is_same<decltype(T::next), Ptr<T>>::value,
+      "T->next must be a Ptr<T>");
 
-    T* head = Terminator();
-    T* tail = Terminator();
+    Ptr<T> head = Terminator();
+    Ptr<T> tail = Terminator();
 
   public:
     ~DLList()
@@ -101,19 +59,19 @@ namespace snmalloc
       return head == Terminator();
     }
 
-    SNMALLOC_FAST_PATH T* get_head()
+    SNMALLOC_FAST_PATH Ptr<T> get_head()
     {
       return head;
     }
 
-    T* get_tail()
+    Ptr<T> get_tail()
     {
       return tail;
     }
 
-    SNMALLOC_FAST_PATH T* pop()
+    SNMALLOC_FAST_PATH Ptr<T> pop()
     {
-      T* item = head;
+      Ptr<T> item = head;
 
       if (item != Terminator())
         remove(item);
@@ -121,9 +79,9 @@ namespace snmalloc
       return item;
     }
 
-    T* pop_tail()
+    Ptr<T> pop_tail()
     {
-      T* item = tail;
+      Ptr<T> item = tail;
 
       if (item != Terminator())
         remove(item);
@@ -131,7 +89,7 @@ namespace snmalloc
       return item;
     }
 
-    void insert(T* item)
+    void insert(Ptr<T> item)
     {
 #ifndef NDEBUG
       debug_check_not_contains(item);
@@ -151,7 +109,7 @@ namespace snmalloc
 #endif
     }
 
-    void insert_back(T* item)
+    void insert_back(Ptr<T> item)
     {
 #ifndef NDEBUG
       debug_check_not_contains(item);
@@ -171,7 +129,7 @@ namespace snmalloc
 #endif
     }
 
-    SNMALLOC_FAST_PATH void remove(T* item)
+    SNMALLOC_FAST_PATH void remove(Ptr<T> item)
     {
 #ifndef NDEBUG
       debug_check_contains(item);
@@ -202,11 +160,11 @@ namespace snmalloc
       }
     }
 
-    void debug_check_contains(T* item)
+    void debug_check_contains(Ptr<T> item)
     {
 #ifndef NDEBUG
       debug_check();
-      T* curr = head;
+      Ptr<T> curr = head;
 
       while (curr != item)
       {
@@ -218,11 +176,11 @@ namespace snmalloc
 #endif
     }
 
-    void debug_check_not_contains(T* item)
+    void debug_check_not_contains(Ptr<T> item)
     {
 #ifndef NDEBUG
       debug_check();
-      T* curr = head;
+      Ptr<T> curr = head;
 
       while (curr != Terminator())
       {
@@ -237,8 +195,8 @@ namespace snmalloc
     void debug_check()
     {
 #ifndef NDEBUG
-      T* item = head;
-      T* prev = Terminator();
+      Ptr<T> item = head;
+      Ptr<T> prev = Terminator();
 
       while (item != Terminator())
       {
