@@ -3,6 +3,8 @@
 #include "bits.h"
 #include "flaglock.h"
 
+#include <type_traits>
+
 namespace snmalloc
 {
   /*
@@ -73,18 +75,26 @@ namespace snmalloc
   template<size_t length, typename T>
   class ModArray
   {
+    /**
+     * Align the elements, so that access is cheaper.
+     */
+    struct alignas(bits::next_pow2_const(sizeof(T))) TWrap
+    {
+      T v;
+    };
+
     static constexpr size_t rlength = bits::next_pow2_const(length);
-    T array[rlength];
+    TWrap array[rlength];
 
   public:
     constexpr const T& operator[](const size_t i) const
     {
-      return array[i & (rlength - 1)];
+      return array[i & (rlength - 1)].v;
     }
 
     constexpr T& operator[](const size_t i)
     {
-      return array[i & (rlength - 1)];
+      return array[i & (rlength - 1)].v;
     }
   };
 
@@ -143,9 +153,74 @@ namespace snmalloc
     };
   };
 
-  template<class T>
-  void ignore(T* t)
+  template<class T, template<typename> typename Ptr>
+  void ignore(Ptr<T> t)
   {
     UNUSED(t);
   }
+
+  /**
+   * Sometimes we need atomics with trivial initializer.  Unfortunately, this
+   * became harder to accomplish in C++20.  Fortunately, our rules for accessing
+   * these are at least as strong as those required by C++20's atomic_ref:
+   *
+   *   * The objects outlive any references to them
+   *
+   *   * We always access the objects through references (though we'd be allowed
+   *     to access them without if we knew there weren't other references)
+   *
+   *   * We don't access sub-objects at all, much less concurrently through
+   *     other references.
+   */
+  template<typename T>
+  class TrivialInitAtomic
+  {
+    static_assert(
+      std::is_trivially_default_constructible_v<T>,
+      "TrivialInitAtomic should not attempt to call nontrivial constructors");
+
+#ifdef __cpp_lib_atomic_ref
+    using Val = T;
+    using Ref = std::atomic_ref<T>;
+#else
+    using Val = std::atomic<T>;
+    using Ref = std::atomic<T>&;
+#endif
+    Val v;
+
+  public:
+    /**
+     * Construct a reference to this value; use .load and .store to manipulate
+     * the value.
+     */
+    SNMALLOC_FAST_PATH Ref ref()
+    {
+#ifdef __cpp_lib_atomic_ref
+      return std::atomic_ref<T>(this->v);
+#else
+      return this->v;
+#endif
+    }
+
+    SNMALLOC_FAST_PATH T
+    load(std::memory_order mo = std::memory_order_seq_cst) noexcept
+    {
+      return this->ref().load(mo);
+    }
+
+    SNMALLOC_FAST_PATH void
+    store(T n, std::memory_order mo = std::memory_order_seq_cst) noexcept
+    {
+      return this->ref().store(n, mo);
+    }
+
+    SNMALLOC_FAST_PATH bool compare_exchange_strong(
+      T& exp, T des, std::memory_order mo = std::memory_order_seq_cst) noexcept
+    {
+      return this->ref().compare_exchange_strong(exp, des, mo);
+    }
+  };
+
+  static_assert(sizeof(TrivialInitAtomic<char>) == sizeof(char));
+  static_assert(alignof(TrivialInitAtomic<char>) == alignof(char));
 } // namespace snmalloc
