@@ -2,6 +2,7 @@
 
 #include "../ds/bits.h"
 #include "../ds/helpers.h"
+#include "../ds/invalidptr.h"
 
 #include <atomic>
 #include <utility>
@@ -52,8 +53,24 @@ namespace snmalloc
    * allocators do not interact with this directly but rather via the
    * static ChunkMap object, which encapsulates knowledge about the
    * pagemap's parametric type T.
+   *
+   * The other template paramters are...
+   *
+   *   GRANULARITY_BITS: the log2 of the size in bytes of the address space
+   *   granule associated with each entry.
+   *
+   *   default_content: An initial value of T (typically "0" or something akin)
+   *
+   *   PrimAlloc: A class used to source PageMap-internal memory; it must have a
+   *   method callable as if it had the following type:
+   *
+   *      template<typename T, size_t alignment> static T* alloc_chunk(void);
    */
-  template<size_t GRANULARITY_BITS, typename T, T default_content>
+  template<
+    size_t GRANULARITY_BITS,
+    typename T,
+    T default_content,
+    typename PrimAlloc>
   class Pagemap
   {
   private:
@@ -90,12 +107,20 @@ namespace snmalloc
 
     struct Leaf
     {
-      std::atomic<T> values[ENTRIES_PER_LEAF];
+      TrivialInitAtomic<T> values[ENTRIES_PER_LEAF];
+
+      static_assert(sizeof(TrivialInitAtomic<T>) == sizeof(T));
+      static_assert(alignof(TrivialInitAtomic<T>) == alignof(T));
     };
 
     struct PagemapEntry
     {
-      std::atomic<PagemapEntry*> entries[ENTRIES_PER_INDEX_LEVEL];
+      TrivialInitAtomic<PagemapEntry*> entries[ENTRIES_PER_INDEX_LEVEL];
+
+      static_assert(
+        sizeof(TrivialInitAtomic<PagemapEntry*>) == sizeof(PagemapEntry*));
+      static_assert(
+        alignof(TrivialInitAtomic<PagemapEntry*>) == alignof(PagemapEntry*));
     };
 
     static_assert(
@@ -110,11 +135,11 @@ namespace snmalloc
     // allocation.
     //  TODO: This is fragile that it is not being memset, and we should review
     //  to ensure we don't get bitten by this in the future.
-    std::atomic<PagemapEntry*> top[TOPLEVEL_ENTRIES]; // = {nullptr};
+    TrivialInitAtomic<PagemapEntry*> top[TOPLEVEL_ENTRIES];
 
     template<bool create_addr>
     SNMALLOC_FAST_PATH PagemapEntry*
-    get_node(std::atomic<PagemapEntry*>* e, bool& result)
+    get_node(TrivialInitAtomic<PagemapEntry*>* e, bool& result)
     {
       // The page map nodes are all allocated directly from the OS zero
       // initialised with a system call.  We don't need any ordered to guarantee
@@ -139,7 +164,7 @@ namespace snmalloc
     }
 
     SNMALLOC_SLOW_PATH PagemapEntry*
-    get_node_slow(std::atomic<PagemapEntry*>* e, bool& result)
+    get_node_slow(TrivialInitAtomic<PagemapEntry*>* e, bool& result)
     {
       // The page map nodes are all allocated directly from the OS zero
       // initialised with a system call.  We don't need any ordered to guarantee
@@ -153,8 +178,7 @@ namespace snmalloc
         if (e->compare_exchange_strong(
               value, LOCKED_ENTRY, std::memory_order_relaxed))
         {
-          auto& v = default_memory_provider();
-          value = v.alloc_chunk<PagemapEntry, OS_PAGE_SIZE>();
+          value = PrimAlloc::template alloc_chunk<PagemapEntry, OS_PAGE_SIZE>();
           e->store(value, std::memory_order_release);
         }
         else
@@ -182,7 +206,7 @@ namespace snmalloc
 #endif
       size_t ix = addr >> TOPLEVEL_SHIFT;
       size_t shift = TOPLEVEL_SHIFT;
-      std::atomic<PagemapEntry*>* e = &top[ix];
+      TrivialInitAtomic<PagemapEntry*>* e = &top[ix];
 
       // This is effectively a
       //   for (size_t i = 0; i < INDEX_LEVELS; i++)
@@ -226,13 +250,14 @@ namespace snmalloc
     }
 
     template<bool create_addr>
-    SNMALLOC_FAST_PATH std::atomic<T>* get_addr(uintptr_t p, bool& success)
+    SNMALLOC_FAST_PATH TrivialInitAtomic<T>*
+    get_addr(uintptr_t p, bool& success)
     {
       auto leaf_ix = get_leaf_index<create_addr>(p, success);
       return &(leaf_ix.first->values[leaf_ix.second]);
     }
 
-    std::atomic<T>* get_ptr(uintptr_t p)
+    TrivialInitAtomic<T>* get_ptr(uintptr_t p)
     {
       bool success;
       return get_addr<true>(p, success);
@@ -317,7 +342,7 @@ namespace snmalloc
         for (; ix < last; ix++)
         {
           SNMALLOC_ASSUME(leaf_ix.first != nullptr);
-          leaf_ix.first->values[ix] = x;
+          leaf_ix.first->values[ix].store(x);
         }
 
         length = length - diff;
@@ -339,7 +364,10 @@ namespace snmalloc
     static constexpr size_t ENTRIES = 1ULL << COVERED_BITS;
     static constexpr size_t SHIFT = GRANULARITY_BITS;
 
-    std::atomic<T> top[ENTRIES];
+    TrivialInitAtomic<T> top[ENTRIES];
+
+    static_assert(sizeof(TrivialInitAtomic<T>) == sizeof(T));
+    static_assert(alignof(TrivialInitAtomic<T>) == alignof(T));
 
   public:
     /**
@@ -409,4 +437,85 @@ namespace snmalloc
         reinterpret_cast<uintptr_t>(&top[p >> SHIFT]) & ~(OS_PAGE_SIZE - 1));
     }
   };
+
+  /**
+   * Mixin used by `ChunkMap` and other `PageMap` consumers to directly access
+   * the pagemap via a global variable.  This should be used from within the
+   * library or program that owns the pagemap.
+   *
+   * This class makes the global pagemap a static field so that its name
+   * includes the type mangling.  If two compilation units try to instantiate
+   * two different types of pagemap then they will see two distinct pagemaps.
+   * This will prevent allocating with one and freeing with the other (because
+   * the memory will show up as not owned by any allocator in the other
+   * compilation unit) but will prevent the same memory being interpreted as
+   * having two different types.
+   *
+   * Simiarly, perhaps two modules wish to instantiate *different* pagemaps
+   * of the *same* type.  Therefore, we add a `Purpose` parameter that can be
+   * used to pry symbols apart.  By default, the `Purpose` is just the type of
+   * the pagemap; that is, pagemaps default to discrimination solely by their
+   * type.
+   */
+  template<typename T, typename Purpose = T>
+  class GlobalPagemapTemplate
+  {
+    /**
+     * The global pagemap variable.  The name of this symbol will include the
+     * type of `T` and `U`.
+     */
+    inline static T global_pagemap;
+
+  public:
+    /**
+     * Returns the pagemap.
+     */
+    static T& pagemap()
+    {
+      return global_pagemap;
+    }
+  };
+
+  /**
+   * Mixin used by `ChunkMap` and other `PageMap` consumers to access the global
+   * pagemap via a type-checked C interface.  This should be used when another
+   * library (e.g.  your C standard library) uses snmalloc and you wish to use a
+   * different configuration in your program or library, but wish to share a
+   * pagemap so that either version can deallocate memory.
+   *
+   * The `Purpose` parameter is as with `GlobalPgemapTemplate`.
+   */
+  template<
+    typename T,
+    void* (*raw_get)(const PagemapConfig**),
+    typename Purpose = T>
+  class ExternalGlobalPagemapTemplate
+  {
+    /**
+     * A pointer to the pagemap.
+     */
+    inline static T* external_pagemap;
+
+  public:
+    /**
+     * Returns the exported pagemap.
+     * Accesses the pagemap via the C ABI accessor and casts it to
+     * the expected type, failing in cases of ABI mismatch.
+     */
+    static T& pagemap()
+    {
+      if (external_pagemap == nullptr)
+      {
+        const snmalloc::PagemapConfig* c = nullptr;
+        void* raw_pagemap = raw_get(&c);
+        external_pagemap = T::cast_to_pagemap(raw_pagemap, c);
+        if (!external_pagemap)
+        {
+          Pal::error("Incorrect ABI of global pagemap.");
+        }
+      }
+      return *external_pagemap;
+    }
+  };
+
 } // namespace snmalloc
