@@ -27,7 +27,7 @@ extern "C"
   void SNMALLOC_NAME_MANGLE(check_start)(void* ptr)
   {
 #if !defined(NDEBUG) && !defined(SNMALLOC_PASS_THROUGH)
-    if (Alloc::external_pointer<Start>(ptr) != ptr)
+    if (ThreadAlloc::get_noncachable()->external_pointer<Start>(ptr) != ptr)
     {
       error("Using pointer that is not to the start of an allocation");
     }
@@ -38,7 +38,7 @@ extern "C"
 
   SNMALLOC_EXPORT void* SNMALLOC_NAME_MANGLE(__malloc_end_pointer)(void* ptr)
   {
-    return Alloc::external_pointer<OnePastEnd>(ptr);
+    return ThreadAlloc::get_noncachable()->external_pointer<OnePastEnd>(ptr);
   }
 
   SNMALLOC_EXPORT void* SNMALLOC_NAME_MANGLE(malloc)(size_t size)
@@ -73,7 +73,7 @@ extern "C"
   size_t SNMALLOC_NAME_MANGLE(malloc_usable_size)(
     MALLOC_USABLE_SIZE_QUALIFIER void* ptr)
   {
-    return Alloc::alloc_size(ptr);
+    return ThreadAlloc::get_noncachable()->alloc_size(ptr);
   }
 
   SNMALLOC_EXPORT void* SNMALLOC_NAME_MANGLE(realloc)(void* ptr, size_t size)
@@ -95,7 +95,7 @@ extern "C"
 
     SNMALLOC_NAME_MANGLE(check_start)(ptr);
 
-    size_t sz = Alloc::alloc_size(ptr);
+    size_t sz = ThreadAlloc::get_noncachable()->alloc_size(ptr);
     // Keep the current allocation if the given size is in the same sizeclass.
     if (sz == round_size(size))
     {
@@ -221,10 +221,10 @@ extern "C"
    * this function to the correct type.  This allows us to preserve some
    * semblance of ABI safety via a pure C API.
    */
-  SNMALLOC_EXPORT void* SNMALLOC_NAME_MANGLE(snmalloc_pagemap_global_get)(
+  SNMALLOC_EXPORT void* SNMALLOC_NAME_MANGLE(snmalloc_chunkmap_global_get)(
     PagemapConfig const** config)
   {
-    auto& pm = GlobalPagemap::pagemap();
+    auto& pm = GlobalChunkmap::pagemap();
     if (config)
     {
       *config = &ChunkmapPagemap::config;
