@@ -61,20 +61,55 @@ namespace snmalloc
     DefaultPal;
 #endif
 
-  [[noreturn]] SNMALLOC_SLOW_PATH inline void error(const char* const str)
+  [[noreturn]] SNMALLOC_SLOW_PATH inline SNMALLOC_COLD void
+  error(const char* const str)
   {
     Pal::error(str);
   }
 
-  /**
-   * Query whether the PAL supports a specific feature.
-   */
-  template<PalFeatures F, SNMALLOC_CONCEPT(ConceptPAL) PAL = Pal>
-  constexpr static bool pal_supports = (PAL::pal_features & F) == F;
-
   // Used to keep Superslab metadata committed.
   static constexpr size_t OS_PAGE_SIZE = Pal::page_size;
 
+  /**
+   * Perform platform-specific adjustment of return pointers.
+   *
+   * This is here, rather than in every PAL proper, merely to minimize
+   * disruption to PALs for platforms that do not support StrictProvenance AALs.
+   */
+  template<typename PAL = Pal, typename AAL = Aal, typename T, capptr_bounds B>
+  static SNMALLOC_FAST_PATH typename std::enable_if_t<
+    !aal_supports<StrictProvenance, AAL>,
+    CapPtr<T, capptr_export_type<B>()>>
+  capptr_export(CapPtr<T, B> p)
+  {
+    return CapPtr<T, capptr_export_type<B>()>(p.unsafe_capptr);
+  }
+
+  template<typename PAL = Pal, typename AAL = Aal, typename T, capptr_bounds B>
+  static SNMALLOC_FAST_PATH typename std::enable_if_t<
+    aal_supports<StrictProvenance, AAL>,
+    CapPtr<T, capptr_export_type<B>()>>
+  capptr_export(CapPtr<T, B> p)
+  {
+    return PAL::capptr_export(p);
+  }
+
+  /**
+   * A convenience wrapper that avoids the need to litter unsafe accesses with
+   * every call to PAL::zero.
+   *
+   * We do this here rather than plumb CapPtr further just to minimize
+   * disruption and avoid code bloat.  This wrapper ought to compile down to
+   * nothing if SROA is doing its job.
+   */
+  template<typename PAL, bool page_aligned = false, typename T, capptr_bounds B>
+  static SNMALLOC_FAST_PATH void pal_zero(CapPtr<T, B> p, size_t sz)
+  {
+    static_assert(
+      !page_aligned || B == CBArena || B == CBChunkD || B == CBChunk);
+    PAL::template zero<page_aligned>(p.unsafe_capptr, sz);
+  }
+
   static_assert(
     bits::is_pow2(OS_PAGE_SIZE), "OS_PAGE_SIZE must be a power of two");
   static_assert(
