@@ -1,6 +1,18 @@
 #pragma once
 
 #if defined(__HAIKU__)
+#  include <cassert>
+
+namespace snmalloc
+{
+  template<typename... Args>
+  int getentropy(Args...)
+  {
+    assert(0 && "Unreachable path");
+    return -1;
+  }
+}
+
 #  include "pal_posix.h"
 
 #  include <sys/mman.h>
@@ -18,8 +30,11 @@ namespace snmalloc
      * Bitmap of PalFeatures flags indicating the optional features that this
      * PAL supports.
      *
+     * Disable the POSIX default implementation of Entropy as not supported on
+     * Haiku.
      */
-    static constexpr uint64_t pal_features = PALPOSIX::pal_features;
+    static constexpr uint64_t pal_features =
+      PALPOSIX::pal_features & ~(Entropy);
 
     /**
      * Haiku requires an explicit no-reserve flag in `mmap` to guarantee lazy
