@@ -4,6 +4,7 @@
 #if defined(BACKTRACE_HEADER)
 #  include BACKTRACE_HEADER
 #endif
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -11,6 +12,12 @@
 #include <sys/mman.h>
 #include <unistd.h>
 #include <utility>
+#if __has_include(<sys/random.h>)
+#  include <sys/random.h>
+#endif
+#if __has_include(<unistd.h>)
+#  include <unistd.h>
+#endif
 
 extern "C" int puts(const char* str);
 
@@ -89,6 +96,22 @@ namespace snmalloc
       static const int fd = T::anonymous_memory_fd;
     };
 
+    /**
+     * A RAII class to capture and restore errno
+     */
+    class KeepErrno
+    {
+      decltype(errno) cached_errno;
+
+    public:
+      KeepErrno() : cached_errno(errno) {}
+
+      ~KeepErrno()
+      {
+        errno = cached_errno;
+      }
+    };
+
   public:
     /**
      * Bitmap of PalFeatures flags indicating the optional features that this
@@ -96,7 +119,7 @@ namespace snmalloc
      *
      * POSIX systems are assumed to support lazy commit.
      */
-    static constexpr uint64_t pal_features = LazyCommit;
+    static constexpr uint64_t pal_features = LazyCommit | Entropy;
 
     static constexpr size_t page_size = Aal::smallest_page_size;
 
@@ -186,6 +209,16 @@ namespace snmalloc
       if (page_aligned || is_aligned_block<OS::page_size>(p, size))
       {
         SNMALLOC_ASSERT(is_aligned_block<OS::page_size>(p, size));
+
+        /*
+         * If mmap fails, we're going to fall back to zeroing the memory
+         * ourselves, which is not stellar, but correct.  However, mmap() will
+         * have has left errno nonzero in an effort to explain its MAP_FAILED
+         * result.  Capture its current value and restore it at the end of this
+         * block.
+         */
+        auto hold = KeepErrno();
+
         void* r = mmap(
           p,
           size,
@@ -237,5 +270,31 @@ namespace snmalloc
 
       OS::error("Out of memory");
     }
+
+    /**
+     * Source of Entropy
+     *
+     * This is a default that works on many POSIX platforms.
+     */
+    static uint64_t get_entropy64()
+    {
+      if constexpr (!pal_supports<Entropy, OS>)
+      {
+        // Derived Pal does not provide entropy.
+        return 0;
+      }
+      else if constexpr (OS::get_entropy64 != get_entropy64)
+      {
+        // Derived Pal has provided a custom definition.
+        return OS::get_entropy64();
+      }
+      else
+      {
+        uint64_t result;
+        if (getentropy(&result, sizeof(result)) != 0)
+          error("Failed to get system randomness");
+        return result;
+      }
+    }
   };
 } // namespace snmalloc
