@@ -2,6 +2,7 @@
 
 int main()
 {
+#ifndef SNMALLOC_PASS_THROUGH // This test depends on snmalloc internals
   snmalloc::Alloc* a = snmalloc::ThreadAlloc::get();
   bool result;
 
@@ -36,4 +37,5 @@ int main()
   {
     abort();
   }
+#endif
 }
\ No newline at end of file
