@@ -33,7 +33,7 @@ namespace test
       // store object
       objects[i] = (size_t*)alloc->alloc(size);
       // Store allocators size for this object
-      *objects[i] = Alloc::alloc_size(objects[i]);
+      *objects[i] = alloc->alloc_size(objects[i]);
     }
   }
 
@@ -64,7 +64,9 @@ namespace test
 #endif
     setup(r, alloc);
 
-    DO_TIME("External pointer queries ", {
+    {
+      MeasureTime m;
+      m << "External pointer queries ";
       for (size_t i = 0; i < iterations; i++)
       {
         size_t rand = (size_t)r.next();
@@ -73,11 +75,11 @@ namespace test
         size_t size = *external_ptr;
         size_t offset = (size >> 4) * (rand & 15);
         void* interior_ptr = pointer_offset(external_ptr, offset);
-        void* calced_external = Alloc::external_pointer(interior_ptr);
+        void* calced_external = alloc->external_pointer(interior_ptr);
         if (calced_external != external_ptr)
           abort();
       }
-    });
+    }
 
     teardown(alloc);
   }
